==> vector_map_tiles-1.2.44.tar.gz: lib/src/grid/tile_model.dart <==
import 'dart:ui' as ui;

import 'package:flutter/widgets.dart';
import '../executor/executor.dart';
import 'package:vector_tile_renderer/vector_tile_renderer.dart';

import '../options.dart';
import '../stream/tile_supplier.dart';
import '../tile_identity.dart';
import 'slippy_map_translator.dart';

typedef ZoomScaleFunction = double Function(int tileZoom);
typedef ZoomFunction = double Function();

class VectorTileModel extends ChangeNotifier {
  bool _disposed = false;
  bool get disposed => _disposed;

  final RenderMode renderMode;
  final TileIdentity tile;
  final TileSupplier tileSupplier;
  final Theme theme;
  bool paintBackground;
  final bool showTileDebugInfo;
  final ZoomScaleFunction zoomScaleFunction;
  final ZoomFunction zoomFunction;
  double lastRenderedZoom = double.negativeInfinity;
  double lastRenderedZoomScale = double.negativeInfinity;
  late final TileTranslation defaultTranslation;
  TileTranslation? translation;
  TileTranslation? imageTranslation;
  Tileset? tileset;
  ui.Image? image;
  TileRequest? _tileRequest;

  VectorTileModel(
      this.renderMode,
      this.tileSupplier,
      this.theme,
      this.tile,
      this.zoomScaleFunction,
      this.zoomFunction,
      this.paintBackground,
      this.showTileDebugInfo) {
    defaultTranslation =
        SlippyMapTranslator(tileSupplier.maximumZoom).translate(tile);
  }

  bool get hasData => image != null || tileset != null;

  void startLoading() async {
    final request = TileRequest(
        tileId: tile.normalize(),
        primaryFormat: renderMode == RenderMode.raster
            ? TileFormat.raster
            : TileFormat.vector,
        secondaryFormat:
            renderMode == RenderMode.mixed ? TileFormat.raster : null,
        cancelled: () => _disposed);
    _tileRequest = request;
    final stream = tileSupplier.stream(request);
    try {
      await stream.forEach(_receiveTile);
    } on CancellationException {
      // ignore, expected
    }
  }

  void _receiveTile(Tile received) {
    final newTranslation = SlippyMapTranslator(tileSupplier.maximumZoom)
        .specificZoomTranslation(tile, zoom: received.identity.z);
    if (received.format == TileFormat.raster) {
      if (_disposed) {
        received.image?.dispose();
      } else {
        var hadImage = image != null;
        image?.dispose();
        image = received.image;
        imageTranslation = newTranslation;
        if (hadImage && renderMode != RenderMode.raster) {
          Future.delayed(Duration(milliseconds: 300)).then((value) {
            if (tileset == null && imageTranslation == newTranslation) {
              notifyListeners();
            }
          });
        } else if (tileset == null) {
          notifyListeners();
        }
      }
    } else {
      tileset = received.tileset;
      translation = newTranslation;
      notifyListeners();
    }
  }

  bool updateRendering() {
    final changed = hasChanged();
    if (changed) {
      lastRenderedZoom = zoomFunction();
      lastRenderedZoomScale = zoomScaleFunction(tile.z);
    }
    return changed;
  }

  bool hasChanged() {
    final lastRenderedZoom = zoomFunction();
    final lastRenderedZoomScale = zoomScaleFunction(tile.z);
    return lastRenderedZoomScale != this.lastRenderedZoomScale ||
        lastRenderedZoom != this.lastRenderedZoom;
  }

  void requestRepaint() {
    notifyListeners();
  }

  @override
  void notifyListeners() {
    if (!_disposed) {
      super.notifyListeners();
    }
  }

  @override
  void dispose() {
    if (!_disposed) {
      super.dispose();
      image?.dispose();
      image = null;
      tileset = null;
      _disposed = true;
    }
  }

  @override
  void removeListener(ui.VoidCallback listener) {
    if (!_disposed) {
      super.removeListener(listener);
    }
  }
}

==> vector_map_tiles-1.3.45.tar.gz: lib/src/grid/tile_model.dart <==
import 'dart:ui' as ui;

import 'package:flutter/widgets.dart';
import '../executor/executor.dart';
import 'package:vector_tile_renderer/vector_tile_renderer.dart';

import '../options.dart';
import '../stream/tile_supplier.dart';
import '../tile_identity.dart';
import 'slippy_map_translator.dart';

typedef ZoomScaleFunction = double Function(int tileZoom);
typedef ZoomFunction = double Function();

class VectorTileModel extends ChangeNotifier {
  bool _disposed = false;
  bool get disposed => _disposed;

  final RenderMode renderMode;
  final TileIdentity tile;
  final TileSupplier tileSupplier;
  final Theme theme;
  bool paintBackground;
  final bool showTileDebugInfo;
  final ZoomScaleFunction zoomScaleFunction;
  final ZoomFunction zoomFunction;
  double lastRenderedZoom = double.negativeInfinity;
  double lastRenderedZoomScale = double.negativeInfinity;
  late final TileTranslation defaultTranslation;
  TileTranslation? translation;
  TileTranslation? imageTranslation;
  Tileset? tileset;
  ui.Image? image;

  VectorTileModel(
      this.renderMode,
      this.tileSupplier,
      this.theme,
      this.tile,
      this.zoomScaleFunction,
      this.zoomFunction,
      this.paintBackground,
      this.showTileDebugInfo) {
    defaultTranslation =
        SlippyMapTranslator(tileSupplier.maximumZoom).translate(tile);
  }

  bool get hasData => image != null || tileset != null;

  void startLoading() async {
    final request = TileRequest(
        tileId: tile.normalize(),
        primaryFormat: renderMode == RenderMode.raster
            ? TileFormat.raster
            : TileFormat.vector,
        secondaryFormat:
            renderMode == RenderMode.mixed ? TileFormat.raster : null,
        cancelled: () => _disposed);
    final stream = tileSupplier.stream(request);
    try {
      await stream.forEach(_receiveTile);
    } on CancellationException {
      // ignore, expected
    }
  }

  void _receiveTile(TileResponse received) {
    final newTranslation = SlippyMapTranslator(tileSupplier.maximumZoom)
        .specificZoomTranslation(tile, zoom: received.identity.z);
    if (received.format == TileFormat.raster) {
      if (_disposed) {
        received.image?.dispose();
      } else {
        var hadImage = image != null;
        image?.dispose();
        image = received.image;
        imageTranslation = newTranslation;
        if (hadImage && renderMode != RenderMode.raster) {
          Future.delayed(Duration(milliseconds: 300)).then((value) {
            if (tileset == null && imageTranslation == newTranslation) {
              notifyListeners();
            }
          });
        } else if (tileset == null) {
          notifyListeners();
        }
      }
    } else {
      tileset = received.tileset;
      translation = newTranslation;
      notifyListeners();
    }
  }

  bool updateRendering() {
    final changed = hasChanged();
    if (changed) {
      lastRenderedZoom = zoomFunction();
      lastRenderedZoomScale = zoomScaleFunction(tile.z);
    }
    return changed;
  }

  bool hasChanged() {
    final lastRenderedZoom = zoomFunction();
    final lastRenderedZoomScale = zoomScaleFunction(tile.z);
    return lastRenderedZoomScale != this.lastRenderedZoomScale ||
        lastRenderedZoom != this.lastRenderedZoom;
  }

  void requestRepaint() {
    notifyListeners();
  }

  @override
  void notifyListeners() {
    if (!_disposed) {
      super.notifyListeners();
    }
  }

  @override
  void dispose() {
    if (!_disposed) {
      super.dispose();
      image?.dispose();
      image = null;
      tileset = null;
      _disposed = true;
    }
  }

  @override
  void removeListener(ui.VoidCallback listener) {
    if (!_disposed) {
      super.removeListener(listener);
    }
  }
}
